/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSAffineTransform;

import dstep.appkit.NSBezierPath;
import dstep.appkit.NSFileWrapper;
import dstep.appkit.NSTextList;
import dstep.appkit.NSTextTable;
import dstep.applicationservices.coregraphics.CGBase;
import dstep.foundation.NSAffineTransform;
import dstep.foundation.NSCoder;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

struct NSAffineTransformStruct
{
	CGFloat m11;
	CGFloat m12;
	CGFloat m21;
	CGFloat m22;
	CGFloat tX;
	CGFloat tY;
}

const TNSAppKitAdditons = `

	NSBezierPath transformBezierPath (NSBezierPath aPath)
	{
		return invokeObjcSelf!(NSBezierPath, "transformBezierPath:", NSBezierPath)(aPath);
	}

	void set ()
	{
		return invokeObjcSelf!(void, "set");
	}

	void concat ()
	{
		return invokeObjcSelf!(void, "concat");
	}

	//mixin ObjcBindMethod!(transformBezierPath, "transformBezierPath:");
	//mixin ObjcBindMethod!(set, "set");
	//mixin ObjcBindMethod!(concat, "concat");

`;

class NSAffineTransform : NSObject, INSCopying, INSCoding
{
	mixin (ObjcWrap);

	static NSAffineTransform transform ()
	{
		return invokeObjcSelfClass!(NSAffineTransform, "transform");
	}

	NSAffineTransform initWithTransform (NSAffineTransform transform)
	{
		id result = invokeObjcSelf!(id, "initWithTransform:", NSAffineTransform)(transform);
		return result is this.objcObject ? this : (result !is null ? new NSAffineTransform(result) : null);
	}

	this (NSAffineTransform transform)
	{
		super(NSAffineTransform.alloc.initWithTransform(transform).objcObject);
	}

	void translateXBy (CGFloat deltaX, CGFloat deltaY)
	{
		return invokeObjcSelf!(void, "translateXBy:yBy:", CGFloat, CGFloat)(deltaX, deltaY);
	}

	void rotateByDegrees (CGFloat angle)
	{
		return invokeObjcSelf!(void, "rotateByDegrees:", CGFloat)(angle);
	}

	void rotateByRadians (CGFloat angle)
	{
		return invokeObjcSelf!(void, "rotateByRadians:", CGFloat)(angle);
	}

	void scaleBy (CGFloat scale)
	{
		return invokeObjcSelf!(void, "scaleBy:", CGFloat)(scale);
	}

	void scaleXBy (CGFloat scaleX, CGFloat scaleY)
	{
		return invokeObjcSelf!(void, "scaleXBy:yBy:", CGFloat, CGFloat)(scaleX, scaleY);
	}

	void invert ()
	{
		return invokeObjcSelf!(void, "invert");
	}

	void appendTransform (NSAffineTransform transform)
	{
		return invokeObjcSelf!(void, "appendTransform:", NSAffineTransform)(transform);
	}

	void prependTransform (NSAffineTransform transform)
	{
		return invokeObjcSelf!(void, "prependTransform:", NSAffineTransform)(transform);
	}

	NSPoint transformPoint (NSPoint aPoint)
	{
		return invokeObjcSelf!(NSPoint, "transformPoint:", NSPoint)(aPoint);
	}

	NSSize transformSize (NSSize aSize)
	{
		return invokeObjcSelf!(NSSize, "transformSize:", NSSize)(aSize);
	}

	NSAffineTransformStruct transformStruct ()
	{
		return invokeObjcSelf!(NSAffineTransformStruct, "transformStruct");
	}

	void setTransformStruct (NSAffineTransformStruct transformStruct)
	{
		return invokeObjcSelf!(void, "setTransformStruct:", NSAffineTransformStruct)(transformStruct);
	}
	
	NSObject copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(NSObject, "copyWithZone:", NSZone*)(zone);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	NSAffineTransform initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(NSAffineTransform, "initWithCoder:", NSCoder)(aDecoder);
	}
	
	// NSAppKitAdditons
	NSBezierPath transformBezierPath (NSBezierPath aPath)
	{
		return invokeObjcSelf!(NSBezierPath, "transformBezierPath:", NSBezierPath)(aPath);
	}

	void set ()
	{
		return invokeObjcSelf!(void, "set");
	}

	void concat ()
	{
		return invokeObjcSelf!(void, "concat");
	}
}